# -*- coding: utf-8 -*-

from amazon_ad.api_v3.base import ZADOpenAPIV3
from amazon_ad.api_v3 import adapters


class SdReportV3(ZADOpenAPIV3):

    def request(self, data: dict):
        path = "/reporting/reports"
        return self.post(path, data)

    def c_campaign_daily(self, start_date: str, end_date: str):
        data = adapters.SDCampaignReportsAdapter(
            group_by=["campaign"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def c_campaign_summary(self, start_date: str, end_date: str):
        data = adapters.SDCampaignReportsAdapter(
            group_by=["campaign"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def c_matched_target_daily(self, start_date: str, end_date: str):
        data = adapters.SDCampaignReportsAdapter(
            group_by=["matchedTarget"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def c_matched_target_summary(self, start_date: str, end_date: str):
        data = adapters.SDCampaignReportsAdapter(
            group_by=["matchedTarget"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def c_campaign_matched_target_daily(self, start_date: str, end_date: str):
        data = adapters.SDCampaignReportsAdapter(
            group_by=["campaign", "matchedTarget"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def c_campaign_matched_target_summary(self, start_date: str, end_date: str):
        data = adapters.SDCampaignReportsAdapter(
            group_by=["campaign", "matchedTarget"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)


    def ag_ad_group_daily(self, start_date: str, end_date: str):
        data = adapters.SDAdGroupReportsAdapter(
            group_by=["adGroup"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def ag_ad_group_summary(self, start_date: str, end_date: str):
        data = adapters.SDAdGroupReportsAdapter(
            group_by=["adGroup"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def ag_matched_target_daily(self, start_date: str, end_date: str):
        data = adapters.SDAdGroupReportsAdapter(
            group_by=["matchedTarget"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def ag_matched_target_summary(self, start_date: str, end_date: str):
        data = adapters.SDAdGroupReportsAdapter(
            group_by=["matchedTarget"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def ag_ad_group_matched_target_daily(self, start_date: str, end_date: str):
        data = adapters.SDAdGroupReportsAdapter(
            group_by=["adGroup", "matchedTarget"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def ag_ad_group_matched_target_summary(self, start_date: str, end_date: str):
        data = adapters.SDAdGroupReportsAdapter(
            group_by=["adGroup", "matchedTarget"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)


    def t_targeting_daily(self, start_date: str, end_date: str):
        data = adapters.SDTargetingReportsAdapter(
            group_by=["targeting"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def t_targeting_summary(self, start_date: str, end_date: str):
        data = adapters.SDTargetingReportsAdapter(
            group_by=["targeting"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def t_matched_target_daily(self, start_date: str, end_date: str):
        data = adapters.SDTargetingReportsAdapter(
            group_by=["matchedTarget"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def t_matched_target_summary(self, start_date: str, end_date: str):
        data = adapters.SDTargetingReportsAdapter(
            group_by=["matchedTarget"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def t_targeting_matched_target_daily(self, start_date: str, end_date: str):
        data = adapters.SDTargetingReportsAdapter(
            group_by=["targeting", "matchedTarget"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def t_targeting_matched_target_summary(self, start_date: str, end_date: str):
        data = adapters.SDTargetingReportsAdapter(
            group_by=["targeting", "matchedTarget"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)


    def ap_advertiser_daily(self, start_date: str, end_date: str):
        data = adapters.SDAdvertisedProductReportsAdapter(
            group_by=["advertiser"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def ap_advertiser_summary(self, start_date: str, end_date: str):
        data = adapters.SDAdvertisedProductReportsAdapter(
            group_by=["advertiser"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)


    def pp_asin_daily(self, start_date: str, end_date: str):
        data = adapters.SDPurchasedProductReportsAdapter(
            group_by=["asin"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def pp_asin_summary(self, start_date: str, end_date: str):
        data = adapters.SDPurchasedProductReportsAdapter(
            group_by=["asin"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)


    def gait_campaign_daily(self, start_date: str, end_date: str):
        data = adapters.SDGrossAndInvalidTrafficReportsAdapter(
            group_by=["campaign"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def gait_campaign_summary(self, start_date: str, end_date: str):
        data = adapters.SDGrossAndInvalidTrafficReportsAdapter(
            group_by=["campaign"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)


# -*- coding: utf-8 -*-

from amazon_ad.api_v3.base import ZADOpenAPIV3
from amazon_ad.api_v3.adapters import SBCampaignReportsAdapter, SBPlacementReportsAdapter, SBAdGroupReportsAdapter, \
    SBAdReportsAdapter, SBTargetingReportsAdapter, SBSearchTermReportsAdapter, SBPurchasedProductReportsAdapter


class SbReportV3(ZADOpenAPIV3):

    def request(self, data: dict):
        path = "/reporting/reports"
        return self.post(path, data)

    def c_campaign_daily(self, start_date: str, end_date: str):
        data = SBCampaignReportsAdapter(
            group_by=["campaign"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def c_campaign_summary(self, start_date: str, end_date: str):
        data = SBCampaignReportsAdapter(
            group_by=["campaign"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def cp_campaign_placement_daily(self, start_date: str, end_date: str):
        data = SBPlacementReportsAdapter(
            group_by=["campaignPlacement"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def cp_campaign_placement_summary(self, start_date: str, end_date: str):
        data = SBPlacementReportsAdapter(
            group_by=["campaignPlacement"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def ag_ad_group_daily(self, start_date: str, end_date: str):
        data = SBAdGroupReportsAdapter(
            group_by=["adGroup"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def ag_ad_group_summary(self, start_date: str, end_date: str):
        data = SBAdGroupReportsAdapter(
            group_by=["adGroup"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def a_ads_daily(self, start_date: str, end_date: str):
        data = SBAdReportsAdapter(
            group_by=["ads"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def a_ads_summary(self, start_date: str, end_date: str):
        data = SBAdReportsAdapter(
            group_by=["ads"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def t_targeting_daily(self, start_date: str, end_date: str):
        data = SBTargetingReportsAdapter(
            group_by=["targeting"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def t_targeting_summary(self, start_date: str, end_date: str):
        data = SBTargetingReportsAdapter(
            group_by=["targeting"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def st_search_term_daily(self, start_date: str, end_date: str):
        data = SBSearchTermReportsAdapter(
            group_by=["searchTerm"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def st_search_term_summary(self, start_date: str, end_date: str):
        data = SBSearchTermReportsAdapter(
            group_by=["searchTerm"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def purchased_asin_daily(self, start_date: str, end_date: str):
        data = SBPurchasedProductReportsAdapter(
            group_by=["purchasedAsin"], time_unit="DAILY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)

    def purchased_asin_summary(self, start_date: str, end_date: str):
        data = SBPurchasedProductReportsAdapter(
            group_by=["purchasedAsin"], time_unit="SUMMARY", start_date=start_date, end_date=end_date,
        ).get_data_raw()
        return self.request(data)



# -*- coding: utf-8 -*-

from amazon_ad.api.base import ZADOpenAPI

ALLOW_REPORT_TYPES = [
    "campaigns",
    "adGroups",
    "targets",
    "keywords",
]


DEFAULT_REPORT_METRICS = {
    "campaigns":  [
        "campaignName",
        "campaignId",
        "campaignStatus",
        "campaignBudget",
        "campaignBudgetType",

        # "campaignRuleBasedBudget",  # V2
        # "applicableBudgetRuleId",  # V2
        # "applicableBudgetRuleName",  # V2
        #
        # "adGroupName",
        # "adGroupId",
        # "keywordText",
        # "keywordBid",
        # "keywordStatus",
        # "targetId",
        #
        # "searchTermImpressionRank",
        # "searchTermImpressionShare"
        #
        # "targetingExpression",
        # "targetingText",
        # "targetingType",
        # "matchType",

        "impressions",
        "clicks",
        "cost",
        "attributedDetailPageViewsClicks14d",
        "attributedSales14d",
        "attributedSales14dSameSKU",
        "attributedConversions14d",
        "attributedConversions14dSameSKU",
        "attributedOrdersNewToBrand14d",
        "attributedOrdersNewToBrandPercentage14d",
        "attributedOrderRateNewToBrand14d",
        "attributedSalesNewToBrand14d",
        "attributedSalesNewToBrandPercentage14d",
        "attributedUnitsOrderedNewToBrand14d",
        "attributedUnitsOrderedNewToBrandPercentage14d",
        "unitsSold14d",
        "dpv14d",

        # "query",
        # "viewableImpressions",
        # "videoFirstQuartileViews",
        # "videoMidpointViews",
        # "videoThirdQuartileViews",
        # "videoCompleteViews",
        # "video5SecondViews",
        # "video5SecondViewRate",
        # "videoUnmutes",
        # "vtr",
        # "vctr",
    ],
    "adGroups": [
        "campaignName",
        "campaignId",
        "campaignStatus",
        "campaignBudget",
        "campaignBudgetType",

        # "campaignRuleBasedBudget",  # V2
        # "applicableBudgetRuleId",  # V2
        # "applicableBudgetRuleName",  # V2
        #
        "adGroupName",
        "adGroupId",
        # "keywordText",
        # "keywordBid",
        # "keywordStatus",
        # "targetId",
        #
        # "searchTermImpressionRank",
        # "searchTermImpressionShare"
        #
        # "targetingExpression",
        # "targetingText",
        # "targetingType",
        # "matchType",

        "impressions",
        "clicks",
        "cost",
        "attributedDetailPageViewsClicks14d",
        "attributedSales14d",
        "attributedSales14dSameSKU",
        "attributedConversions14d",
        "attributedConversions14dSameSKU",
        "attributedOrdersNewToBrand14d",
        "attributedOrdersNewToBrandPercentage14d",
        "attributedOrderRateNewToBrand14d",
        "attributedSalesNewToBrand14d",
        "attributedSalesNewToBrandPercentage14d",
        "attributedUnitsOrderedNewToBrand14d",
        "attributedUnitsOrderedNewToBrandPercentage14d",
        "unitsSold14d",
        "dpv14d",

        # "query",
        # "viewableImpressions",
        # "videoFirstQuartileViews",
        # "videoMidpointViews",
        # "videoThirdQuartileViews",
        # "videoCompleteViews",
        # "video5SecondViews",
        # "video5SecondViewRate",
        # "videoUnmutes",
        # "vtr",
        # "vctr",
    ],

    # Unsupported fields for targets report: keywordStatus,keywordText,keywordBid,matchType.
    # Unrecognized metric: query
    # Unsupported fields for targets report:
    # applicableBudgetRuleId,applicableBudgetRuleName,searchTermImpressionShare,campaignRuleBasedBudget,searchTermImpressionRank.
    # Metric vtr is not available for campaign type HEADLINE_SEARCH
    "targets": [
        "campaignName",
        "campaignId",
        "campaignStatus",
        "campaignBudget",
        "campaignBudgetType",

        # "campaignRuleBasedBudget",  # V2
        # "applicableBudgetRuleId",  # V2
        # "applicableBudgetRuleName",  # V2

        "adGroupName",
        "adGroupId",
        # "keywordText",
        # "keywordBid",
        # "keywordStatus",
        "targetId",

        # "searchTermImpressionRank",  # V2
        # "searchTermImpressionShare",  # V2

        "targetingExpression",
        "targetingText",
        "targetingType",
        # "matchType",
        "impressions",
        "clicks",
        "cost",
        "attributedDetailPageViewsClicks14d",
        "attributedSales14d",
        "attributedSales14dSameSKU",
        "attributedConversions14d",
        "attributedConversions14dSameSKU",
        "attributedOrdersNewToBrand14d",
        "attributedOrdersNewToBrandPercentage14d",
        "attributedOrderRateNewToBrand14d",
        "attributedSalesNewToBrand14d",
        "attributedSalesNewToBrandPercentage14d",
        "attributedUnitsOrderedNewToBrand14d",
        "attributedUnitsOrderedNewToBrandPercentage14d",
        "unitsSold14d",
        "dpv14d",

        # "query",  # V2
        # "viewableImpressions",  # V2
        # "videoFirstQuartileViews",  # V2
        # "videoMidpointViews",  # V2
        # "videoThirdQuartileViews",  # V2
        # "videoCompleteViews",  # V2
        # "video5SecondViews",  # V2
        # "video5SecondViewRate",  # V2
        # "videoUnmutes",  # V2
        # "vtr",  # V2
        # "vctr",  # V2
    ],
    "keywords": [
        "campaignName",
        "campaignId",
        "campaignStatus",
        "campaignBudget",
        "campaignBudgetType",

        # "campaignRuleBasedBudget",  # V2
        # "applicableBudgetRuleId",  # V2
        # "applicableBudgetRuleName",  # V2

        "adGroupName",
        "adGroupId",
        "keywordText",
        "keywordBid",
        "keywordStatus",
        "targetId",

        # "searchTermImpressionRank",
        # "searchTermImpressionShare"

        "targetingExpression",
        "targetingText",
        "targetingType",
        "matchType",
        "impressions",
        "clicks",
        "cost",
        "attributedDetailPageViewsClicks14d",
        "attributedSales14d",
        "attributedSales14dSameSKU",
        "attributedConversions14d",
        "attributedConversions14dSameSKU",
        "attributedOrdersNewToBrand14d",
        "attributedOrdersNewToBrandPercentage14d",
        "attributedOrderRateNewToBrand14d",
        "attributedSalesNewToBrand14d",
        "attributedSalesNewToBrandPercentage14d",
        "attributedUnitsOrderedNewToBrand14d",
        "attributedUnitsOrderedNewToBrandPercentage14d",
        "unitsSold14d",
        "dpv14d",

        # "query",
        # "viewableImpressions",
        # "videoFirstQuartileViews",
        # "videoMidpointViews",
        # "videoThirdQuartileViews",
        # "videoCompleteViews",
        # "video5SecondViews",
        # "video5SecondViewRate",
        # "videoUnmutes",
        # "vtr",
        # "vctr",
    ],
    # 貌似所有sb报表均可以导出这些字段（有的为空值），建议使用快照了导出全字段
    "all": [
        "campaignName",
        "campaignId",
        "campaignStatus",
        "campaignBudget",
        "campaignBudgetType",

        "campaignRuleBasedBudget",  # V2
        "applicableBudgetRuleId",  # V2
        "applicableBudgetRuleName",  # V2

        "adGroupName",
        "adGroupId",
        "keywordText",
        "keywordBid",
        "keywordStatus",
        "targetId",

        "searchTermImpressionRank",
        "searchTermImpressionShare"

        "targetingExpression",
        "targetingText",
        "targetingType",
        "matchType",
        "impressions",
        "clicks",
        "cost",
        "attributedDetailPageViewsClicks14d",
        "attributedSales14d",
        "attributedSales14dSameSKU",
        "attributedConversions14d",
        "attributedConversions14dSameSKU",
        "attributedOrdersNewToBrand14d",
        "attributedOrdersNewToBrandPercentage14d",
        "attributedOrderRateNewToBrand14d",
        "attributedSalesNewToBrand14d",
        "attributedSalesNewToBrandPercentage14d",
        "attributedUnitsOrderedNewToBrand14d",
        "attributedUnitsOrderedNewToBrandPercentage14d",
        "unitsSold14d",
        "dpv14d",

        "query",
        "viewableImpressions",
        "videoFirstQuartileViews",
        "videoMidpointViews",
        "videoThirdQuartileViews",
        "videoCompleteViews",
        "video5SecondViews",
        "video5SecondViewRate",
        "videoUnmutes",
        "vtr",
        "vctr",
    ],

}

# 这些字段存在于 赞助品牌视频报告，即 creativeType=video
AVAILABLE_VIDEO_REPORT_METRICS = [
    "campaignName",
    "campaignId",
    "campaignStatus",
    "campaignBudget",
    "campaignBudgetType",
    "adGroupName",
    "adGroupId",
    "keywordText",
    "keywordBid",
    "keywordStatus",
    "targetId",
    "targetingExpression",
    "targetingText",
    "targetingType",
    "matchType",
    "impressions",
    "clicks",
    "cost",
    "attributedSales14d",
    "attributedSales14dSameSKU",
    "attributedConversions14d",
    "attributedConversions14dSameSKU",

    "query",  # V2
    "viewableImpressions",  # V2
    "videoFirstQuartileViews",  # V2
    "videoMidpointViews",  # V2
    "videoThirdQuartileViews",  # V2
    "videoCompleteViews",  # V2
    "video5SecondViews",  # V2
    "video5SecondViewRate",  # V2
    "videoUnmutes",  # V2
    "vtr",  # V2
    "vctr",  # V2
]


# 这些字段仅在video报表中存在
VIDEO_METRICS = [
    # "query",  # V2
    "viewableImpressions",  # V2
    "videoFirstQuartileViews",  # V2
    "videoMidpointViews",  # V2
    "videoThirdQuartileViews",  # V2
    "videoCompleteViews",  # V2
    "video5SecondViews",  # V2
    "video5SecondViewRate",  # V2
    "videoUnmutes",  # V2
    "vtr",  # V2
    "vctr",  # V2
]


# 这些类型的报表可以按相应的维度来细分出的报表
DEFAULT_REPORT_DIMENSIONAL = {
    "keywords": "query",  # search terms 搜索词
    "campaigns": "placement",  # placement location 展示位置
}

DEFAULT_CREATIVE_TYPE = [
    "video"
]


class SbReport(ZADOpenAPI):
    def request(self, record_type, report_date, metrics, segment=None, creative_type=None):
        """
        POST    /v2/hsa/{recordType}/report

        Request the creation of a performance report for all entities of a single type which have performance data to
        report. Record types can be: campaigns, adGroups, and keywords

        :param record_type: 枚举 ['campaigns', 'adGroups', 'keywords']
        :param report_date: 报表日期, 格式为: YYYYMMDD
        :param metrics: 列表对象，成员为报表指标，即导出字段；若 metrics=None, 则使用默认指标
        :param segment: [可选]
        :param creative_type: [可选]; 枚举 ['video']
        """

        path = '/v2/hsa/{record_type}/report'.format(record_type=record_type)

        if isinstance(metrics, (list, tuple)):
            metrics = ','.join(metrics)

        data = {
           'reportDate': report_date,
           'metrics': metrics
        }

        if segment:
            data['segment'] = segment

        if creative_type:
            data['creativeType'] = creative_type

        return self.post(path, data)

    def campaigns(self, report_date, metrics=None):
        """
        campaigns = set(json.loads(client.sb_report.campaigns(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [19]: len(campaigns)
        Out[19]: 25
        """
        creative_type = None
        if not metrics:
            metrics = [
                'campaignName',
                'campaignId',
                'campaignStatus',
                'campaignBudget',
                'campaignBudgetType',
                'impressions',
                'clicks',
                'cost',
                'attributedDetailPageViewsClicks14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedOrderRateNewToBrand14d',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'unitsSold14d',
                'dpv14d',
                'applicableBudgetRuleId',
                'applicableBudgetRuleName',
                'campaignRuleBasedBudget',
            ]
        return self.request('campaigns', report_date, metrics, creative_type=creative_type)

    def campaigns_video(self, report_date, metrics=None):
        """
        campaigns_video = set(json.loads(client.sb_report.campaigns_video(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [21]: len(campaigns_video)
        Out[21]: 30

        In [43]: campaigns_video - campaigns
        Out[43]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [44]: campaigns - campaigns_video
        Out[44]:
        {'applicableBudgetRuleId',
         'applicableBudgetRuleName',
         'campaignRuleBasedBudget',
         'dpv14d',
         'unitsSold14d'}
        """
        creative_type = 'video'
        if not metrics:
            metrics = [
                'attributedSales14d',
                'campaignStatus',
                'campaignBudget',
                'video5SecondViewRate',
                'cost',
                'videoUnmutes',
                'clicks',
                'attributedSales14dSameSKU',
                'campaignName',
                'attributedConversions14d',
                'videoThirdQuartileViews',
                'videoFirstQuartileViews',
                'videoCompleteViews',
                'vctr',
                'campaignId',
                'impressions',
                'viewableImpressions',
                'attributedConversions14dSameSKU',
                'videoMidpointViews',
                'video5SecondViews',
                'vtr',
                'campaignBudgetType',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
            ]
        return self.request('campaigns', report_date, metrics, creative_type=creative_type)

    def campaigns_all(self, report_date, metrics=None):
        """
        campaigns_all = campaigns | campaigns_video

        In [26]: len(campaigns_all)
        Out[26]: 35

        In [28]: campaigns_all - campaigns
        Out[28]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [29]: campaigns_all - campaigns_video
        Out[29]:
        {'applicableBudgetRuleId',
         'applicableBudgetRuleName',
         'campaignRuleBasedBudget',
         'dpv14d',
         'unitsSold14d'}
        """
        creative_type = 'all'
        if not metrics:
            metrics = [
                'applicableBudgetRuleId',
                'applicableBudgetRuleName',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignRuleBasedBudget',
                'campaignStatus',
                'clicks',
                'cost',
                'dpv14d',
                'impressions',
                'unitsSold14d',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr',

                'attributedBrandedSearches14d',
                'currency',
                'topOfSearchImpressionShare',
            ]

        return self.request('campaigns', report_date, metrics, creative_type=creative_type)

    def placements(self, report_date, metrics=None):
        """
        placements = set(json.loads(client.sb_report.placements(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [33]: placements - campaigns
        Out[33]: set()

        In [34]: campaigns - placements
        Out[34]:
        {'applicableBudgetRuleId',
         'applicableBudgetRuleName',
         'campaignRuleBasedBudget'}
        """
        segment = 'placement'
        creative_type = None
        if not metrics:
            metrics = [
                'campaignName',
                'campaignId',
                'campaignStatus',
                'campaignBudget',
                'campaignBudgetType',
                'impressions',
                'clicks',
                'cost',
                'attributedDetailPageViewsClicks14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedOrderRateNewToBrand14d',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'unitsSold14d',
                'dpv14d',
            ]

        return self.request('campaigns', report_date, metrics, segment=segment, creative_type=creative_type)

    def placements_video(self, report_date, metrics=None):
        """
        placements_video = set(json.loads(client.sb_report.placements_video(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [36]: len(placements_video)
        Out[36]: 30

        In [37]: placements_video - placements
        Out[37]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [38]: placements - placements_video
        Out[38]: {'dpv14d', 'unitsSold14d'}

        """
        segment = 'placement'
        creative_type = 'video'
        if not metrics:
            metrics = [
                'attributedSales14d',
                'campaignStatus',
                'campaignBudget',
                'video5SecondViewRate',
                'cost',
                'videoUnmutes',
                'clicks',
                'attributedSales14dSameSKU',
                'campaignName',
                'attributedConversions14d',
                'videoThirdQuartileViews',
                'videoFirstQuartileViews',
                'videoCompleteViews',
                'vctr',
                'campaignId',
                'impressions',
                'viewableImpressions',
                'attributedConversions14dSameSKU',
                'videoMidpointViews',
                'video5SecondViews',
                'vtr',
                'campaignBudgetType',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
            ]
        return self.request('campaigns', report_date, metrics, segment=segment, creative_type=creative_type)

    def placements_all(self, report_date, metrics=None):
        """
        placements_all = placements | placements_video

        In [47]: len(placements_all)
        Out[47]: 32

        In [48]: placements_all - placements
        Out[48]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [49]: placements_all - placements_video
        Out[49]: {'dpv14d', 'unitsSold14d'}
        """

        segment = 'placement'
        creative_type = 'all'
        if not metrics:
            metrics = [
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'dpv14d',
                'impressions',
                'unitsSold14d',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr',

                'applicableBudgetRuleId',
                'applicableBudgetRuleName',
                'attributedBrandedSearches14d',
                'campaignRuleBasedBudget',
                'currency',
                'topOfSearchImpressionShare',
            ]
        return self.request('campaigns', report_date, metrics, segment=segment, creative_type=creative_type)

    def ad_groups(self, report_date, metrics=None):
        """
        ad_groups = set(json.loads(client.sb_report.ad_groups(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [54]: len(ad_groups)
        Out[54]: 24
        """
        creative_type = None
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'dpv14d',
                'impressions',
                'unitsSold14d'
            ]
        return self.request('adGroups', report_date, metrics, creative_type=creative_type)

    def ad_groups_video(self, report_date, metrics=None):
        """
        ad_groups_video = set(json.loads(client.sb_report.ad_groups_video(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [59]: len(ad_groups_video)
        Out[59]: 24

        In [60]: ad_groups_video - ad_groups
        Out[60]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [61]: ad_groups - ad_groups_video
        Out[61]:
        {'attributedDetailPageViewsClicks14d',
         'attributedOrderRateNewToBrand14d',
         'attributedOrdersNewToBrand14d',
         'attributedOrdersNewToBrandPercentage14d',
         'attributedSalesNewToBrand14d',
         'attributedSalesNewToBrandPercentage14d',
         'attributedUnitsOrderedNewToBrand14d',
         'attributedUnitsOrderedNewToBrandPercentage14d',
         'dpv14d',
         'unitsSold14d'}
        """
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'impressions',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr']

        creative_type = 'video'
        return self.request('adGroups', report_date, metrics, creative_type=creative_type)

    def ad_groups_all(self, report_date, metrics=None):
        """
        ad_groups_all = ad_groups | ad_groups_video

        In [64]: len(ad_groups_all)
        Out[64]: 34

        In [65]: ad_groups_all - ad_groups
        Out[65]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [66]: ad_groups_all - ad_groups_video
        Out[66]:
        {'attributedDetailPageViewsClicks14d',
         'attributedOrderRateNewToBrand14d',
         'attributedOrdersNewToBrand14d',
         'attributedOrdersNewToBrandPercentage14d',
         'attributedSalesNewToBrand14d',
         'attributedSalesNewToBrandPercentage14d',
         'attributedUnitsOrderedNewToBrand14d',
         'attributedUnitsOrderedNewToBrandPercentage14d',
         'dpv14d',
         'unitsSold14d'}
        """
        creative_type = 'all'
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'dpv14d',
                'impressions',
                'unitsSold14d',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr',

                'attributedBrandedSearches14d',
                'currency',
            ]
        return self.request('adGroups', report_date, metrics, creative_type=creative_type)

    def keywords(self, report_date, metrics=None):
        """
        keywords = set(json.loads(client.sb_report.keywords(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [72]: len(keywords)
        Out[72]: 34
        """
        creative_type = None
        if not metrics:
            metrics = [
                'campaignName',
                'campaignId',
                'campaignStatus',
                'campaignBudget',
                'campaignBudgetType',
                'adGroupName',
                'adGroupId',
                'keywordText',
                'keywordBid',
                'keywordStatus',
                'targetId',
                'targetingExpression',
                'targetingText',
                'targetingType',
                'matchType',
                'impressions',
                'clicks',
                'cost',
                'attributedDetailPageViewsClicks14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedOrderRateNewToBrand14d',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'unitsSold14d',
                'dpv14d',
                'searchTermImpressionRank',
                'searchTermImpressionShare',
            ]
        return self.request('keywords', report_date, metrics, creative_type=creative_type)

    def keywords_video(self, report_date, metrics=None):
        """
        keywords_video = set(json.loads(client.sb_report.keywords_video(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [77]: len(keywords_video)
        Out[77]: 40

        In [78]: keywords_video - keywords
        Out[78]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [84]: keywords - keywords_video
        Out[84]:
        {'dpv14d',
         'searchTermImpressionRank',
         'searchTermImpressionShare',
         'unitsSold14d'}

        """
        creative_type = 'video'
        if not metrics:
            metrics = [
                'attributedSales14d',
                'campaignStatus',
                'campaignBudget',
                'video5SecondViewRate',
                'cost',
                'targetingText',
                'videoUnmutes',
                'clicks',
                'attributedSales14dSameSKU',
                'campaignName',
                'attributedConversions14d',
                'videoThirdQuartileViews',
                'videoFirstQuartileViews',
                'keywordBid',
                'matchType',
                'adGroupName',
                'keywordText',
                'videoCompleteViews',
                'vctr',
                'campaignId',
                'keywordStatus',
                'impressions',
                'targetId',
                'targetingType',
                'viewableImpressions',
                'adGroupId',
                'attributedConversions14dSameSKU',
                'targetingExpression',
                'videoMidpointViews',
                'video5SecondViews',
                'vtr',
                'campaignBudgetType',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
            ]

        return self.request('keywords', report_date, metrics, creative_type=creative_type)

    def keywords_all(self, report_date, metrics=None):
        """
        keywords_all = keywords | keywords_video

        In [81]: len(keywords_all)
        Out[81]: 44

        In [82]: keywords_all - keywords
        Out[82]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [83]: keywords_all - keywords_video
        Out[83]:
        {'dpv14d',
         'searchTermImpressionRank',
         'searchTermImpressionShare',
         'unitsSold14d'}
        """

        creative_type = 'all'
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'dpv14d',
                'impressions',
                'keywordBid',
                'keywordStatus',
                'keywordText',
                'matchType',
                'searchTermImpressionRank',
                'searchTermImpressionShare',
                'targetId',
                'targetingExpression',
                'targetingText',
                'targetingType',
                'unitsSold14d',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr',

                'applicableBudgetRuleId',
                'applicableBudgetRuleName',
                'attributedBrandedSearches14d',
                'campaignRuleBasedBudget',
                'currency',
                'keywordId',
                'topOfSearchImpressionShare',
            ]
        return self.request('keywords', report_date, metrics, creative_type=creative_type)

    def keywords_query(self, report_date, metrics=None):
        """
        keywords_query = set(json.loads(client.sb_report.keywords_query(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [87]: len(keywords_query)
        Out[87]: 16
        """

        segment = 'query'
        creative_type = None
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedSales14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'impressions',
                'keywordBid',
                'keywordStatus',
                'keywordText',
                'matchType',
            ]
        return self.request('keywords', report_date, metrics, segment=segment, creative_type=creative_type)

    def keywords_query_video(self, report_date, metrics=None):
        """
        keywords_query_video = set(json.loads(client.sb_report.keywords_query_video(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [90]: len(keywords_query_video)
        Out[90]: 26

        In [91]: keywords_query_video - keywords_query
        Out[91]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [92]: keywords_query - keywords_query_video
        Out[92]: set()

        """
        segment = 'query'
        creative_type = 'video'
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedSales14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'impressions',
                'keywordBid',
                'keywordStatus',
                'keywordText',
                'matchType',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr',
            ]
        return self.request('keywords', report_date, metrics, segment=segment, creative_type=creative_type)

    def keywords_query_all(self, report_date, metrics=None):
        """
        keywords_query_all = keywords_query | keywords_query_video

        In [95]: len(keywords_query_all)
        Out[95]: 26

        In [96]: keywords_query_all - keywords_query_video
        Out[96]: set()

        In [97]: keywords_query_all - keywords_query
        Out[97]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}
        """
        segment = 'query'
        creative_type = 'all'
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedSales14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'impressions',
                'keywordBid',
                'keywordStatus',
                'keywordText',
                'matchType',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr',

                'keywordId',
                # 'query',
                'searchTermImpressionRank',
                'searchTermImpressionShare',
            ]
        return self.request('keywords', report_date, metrics, segment=segment, creative_type=creative_type)

    def keywords_placement(self, report_date, metrics=None):
        """
        keywords_placement = set(json.loads(client.sb_report.keywords_placement(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [100]: len(keywords_placement)
        Out[100]: 32

        In [102]: keywords_placement - keywords
        Out[102]: set()

        In [103]: keywords - keywords_placement
        Out[103]: {'searchTermImpressionRank', 'searchTermImpressionShare'}
        """

        segment = 'placement'
        creative_type = None
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'dpv14d',
                'impressions',
                'keywordBid',
                'keywordStatus',
                'keywordText',
                'matchType',
                'targetId',  # 实际上拿不到
                'targetingExpression',  # 实际上拿不到
                'targetingText',  # 实际上拿不到
                'targetingType',  # 实际上拿不到
                'unitsSold14d',
                # 'keywordId',  # 不加，默认也能得到
                # 'placement',  # 不能加，实际上就能拿到，但存在为NULL的情况
            ]
        return self.request('keywords', report_date, metrics, segment=segment, creative_type=creative_type)

    def keywords_placement_video(self, report_date, metrics=None):
        """
        keywords_placement_video = set(json.loads(client.sb_report.keywords_placement_video(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [105]: len(keywords_placement_video)
        Out[105]: 32

        In [106]: keywords_placement_video - keywords_placement
        Out[106]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [107]: keywords_placement - keywords_placement_video
        Out[107]:
        {'attributedDetailPageViewsClicks14d',
         'attributedOrderRateNewToBrand14d',
         'attributedOrdersNewToBrand14d',
         'attributedOrdersNewToBrandPercentage14d',
         'attributedSalesNewToBrand14d',
         'attributedSalesNewToBrandPercentage14d',
         'attributedUnitsOrderedNewToBrand14d',
         'attributedUnitsOrderedNewToBrandPercentage14d',
         'dpv14d',
         'unitsSold14d'}
        """

        segment = 'placement'
        creative_type = 'video'
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'impressions',
                'keywordBid',
                'keywordStatus',
                'keywordText',
                'matchType',
                'targetId',
                'targetingExpression',
                'targetingText',
                'targetingType',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr',
            ]
        return self.request('keywords', report_date, metrics, segment=segment, creative_type=creative_type)

    def keywords_placement_all(self, report_date, metrics=None):
        """
        keywords_placement_all = keywords_placement_video | keywords_placement

        In [111]: len(keywords_placement_all)
        Out[111]: 42

        In [112]: keywords_placement_all - keywords_placement
        Out[112]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [113]: keywords_placement_all - keywords_placement_video
        Out[113]:
        {'attributedDetailPageViewsClicks14d',
         'attributedOrderRateNewToBrand14d',
         'attributedOrdersNewToBrand14d',
         'attributedOrdersNewToBrandPercentage14d',
         'attributedSalesNewToBrand14d',
         'attributedSalesNewToBrandPercentage14d',
         'attributedUnitsOrderedNewToBrand14d',
         'attributedUnitsOrderedNewToBrandPercentage14d',
         'dpv14d',
         'unitsSold14d'}
        """
        segment = 'placement'
        creative_type = 'all'
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'dpv14d',
                'impressions',
                'keywordBid',
                'keywordStatus',
                'keywordText',
                'matchType',
                'targetId',
                'targetingExpression',
                'targetingText',
                'targetingType',
                'unitsSold14d',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr',
            ]
        return self.request('keywords', report_date, metrics, segment=segment, creative_type=creative_type)

    def targets(self, report_date, metrics=None):
        """
        targets = set(json.loads(client.sb_report.targets(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [116]: len(targets)
        Out[116]: 28
        """
        creative_type = None
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'dpv14d',
                'impressions',
                'targetId',
                'targetingExpression',
                'targetingText',
                'targetingType',
                'unitsSold14d',
            ]
        return self.request('targets', report_date, metrics, creative_type=creative_type)

    def targets_video(self, report_date, metrics=None):
        """
        targets_video = set(json.loads(client.sb_report.targets_video(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [120]: len(targets_video)
        Out[120]: 36

        In [121]: targets_video - targets
        Out[121]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [122]: targets - targets_video
        Out[122]: {'dpv14d', 'unitsSold14d'}
        """

        creative_type = 'video'
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'impressions',
                'targetId',
                'targetingExpression',
                'targetingText',
                'targetingType',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr',
            ]

        return self.request('targets', report_date, metrics, creative_type=creative_type)

    def targets_all(self, report_date, metrics=None):
        """
        targets_all = targets | targets_video

        In [127]: len(targets_all)
        Out[127]: 38

        In [128]: targets_all - targets
        Out[128]:
        {'vctr',
         'video5SecondViewRate',
         'video5SecondViews',
         'videoCompleteViews',
         'videoFirstQuartileViews',
         'videoMidpointViews',
         'videoThirdQuartileViews',
         'videoUnmutes',
         'viewableImpressions',
         'vtr'}

        In [129]: targets_all - targets_video
        Out[129]: {'dpv14d', 'unitsSold14d'}
        """
        creative_type = 'all'
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignStatus',
                'clicks',
                'cost',
                'dpv14d',
                'impressions',
                'targetId',
                'targetingExpression',
                'targetingText',
                'targetingType',
                'unitsSold14d',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr',

                'attributedBrandedSearches14d',
                'currency',
                'topOfSearchImpressionShare',
            ]
        return self.request('targets', report_date, metrics, creative_type=creative_type)

    def ads_all(self, report_date, metrics=None):
        """
        ads_all = set(json.loads(client.sb_report.ads_all(report_date=report_date)['kwargs']['data'].decode('utf-8'))['metrics'].split(','))

        In [140]: len(ads_all)
        Out[140]: 39
        """
        creative_type = 'all'
        if not metrics:
            metrics = [
                'adGroupId',
                'adGroupName',
                'adId',
                'applicableBudgetRuleId',
                'applicableBudgetRuleName',
                'attributedBrandedSearches14d',
                'attributedConversions14d',
                'attributedConversions14dSameSKU',
                'attributedDetailPageViewsClicks14d',
                'attributedOrderRateNewToBrand14d',
                'attributedOrdersNewToBrand14d',
                'attributedOrdersNewToBrandPercentage14d',
                'attributedSales14d',
                'attributedSales14dSameSKU',
                'attributedSalesNewToBrand14d',
                'attributedSalesNewToBrandPercentage14d',
                'attributedUnitsOrderedNewToBrand14d',
                'attributedUnitsOrderedNewToBrandPercentage14d',
                'campaignBudget',
                'campaignBudgetType',
                'campaignId',
                'campaignName',
                'campaignRuleBasedBudget',
                'campaignStatus',
                'clicks',
                'cost',
                'dpv14d',
                'impressions',
                'unitsSold14d',
                'vctr',
                'video5SecondViewRate',
                'video5SecondViews',
                'videoCompleteViews',
                'videoFirstQuartileViews',
                'videoMidpointViews',
                'videoThirdQuartileViews',
                'videoUnmutes',
                'viewableImpressions',
                'vtr',

                'currency',
            ]
        return self.request('ads', report_date, metrics, creative_type=creative_type)

    def local_test(self, **kwargs):
        """
        测试用的接口
        """
        path = kwargs.get('path')
        data = kwargs.get('data')
        return self.post(path, data)



# -*- coding: utf-8 -*-
"""
https://advertising.amazon.com/API/docs/en-us/amazon-attribution-prod-3p/#/Publishers
"""

from amazon_ad.api.base import ZADOpenAPI


class AttributionPublishers(ZADOpenAPI):

    def get_publishers(self, params=None):
        path = '/attribution/publishers'
        return self.get(path, params=params)



# -*- coding: utf-8 -*-

import copy
import arrow
from pprint import pprint
from enum import Enum


class TimeUnit(Enum):
    """
    请求报告参数、必填、"timeUnit"

    timeUnit can be set to "DAILY" or "SUMMARY".
    If you set timeUnit to "DAILY", you should include "date" in your column list.
    If you set timeUnit to "SUMMARY" you can include "startDate" and "endDate" in your column list.

    https://advertising.amazon.com/API/docs/en-us/reporting/v3/get-started#timeunit-and-supported-columns

    Records:
    1) DAILY = ["date", "startDate", "endDate"]
    {"code":"400","detail":"configuration startDate and endDate are not supported columns for DAILY time unit. Please use date instead."}

    2) SUMMARY = ["startDate", "endDate", "date"]
    {"code":"400","detail":"configuration date is not a supported column for SUMMARY time unit. Please use startDate and/or endDate instead."}
    """
    DAILY = ["date"]
    SUMMARY = ["startDate", "endDate"]

    def __init__(self, columns):
        """Easy dot access like: TimeUnit.SUMMARY.columns"""
        self.columns = columns


class GroupBy(Enum):
    """
    请求报告参数、必填、"groupBy"

    所有报告请求都需要报告配置中的参数。确定报告的粒度级别。如果报告类型支持，您可以在请求中使用多个值。

    https://advertising.amazon.com/API/docs/en-us/reporting/v3/get-started#groupby
    https://advertising.amazon.com/API/docs/en-us/reporting/v3/report-types
    """
    pass


class AdsV3ReportsAdapter(object):

    _report_type_id: str = None
    _ad_product: str = None  # SPONSORED_PRODUCTS | SPONSORED_BRANDS | SPONSORED_DISPLAY
    _maximum_date_range: int = None  # days
    _data_retention: int = None  # days
    _time_unit_choices: tuple = ("SUMMARY", "DAILY")
    _group_by_choices: tuple = None
    _base_metrics: list = None
    _format: str = "GZIP_JSON"

    class GroupBy(Enum):
        def __init__(self, columns):
            self.columns = columns

    def __init__(self,
                 group_by: list,
                 time_unit: str,
                 start_date: str,
                 end_date: str,
                 name: str = None):
        assert set(group_by) in self._group_by_choices
        group_by.sort()
        self.group_by = group_by

        assert time_unit in self._time_unit_choices
        self.time_unit = time_unit

        assert arrow.get(end_date) > arrow.get(start_date), ":::ERROR: end_date < start_date !!!"
        # assert (arrow.get(end_date) - arrow.get(start_date)).days <= self._maximum_date_range, \
        #     "The maximum time range is exceeded"
        self.start_date = start_date  # "2022-07-01"
        self.end_date = end_date  # "2022-07-10"

        if not name:
            name = "%s_%s_%s_%s_%s_%s" % (self._report_type_id, self.start_date, self.end_date, '_'.join(self.group_by),
                                          self.time_unit, arrow.utcnow().isoformat())
        self.name = name

    def get_columns(self):
        columns = copy.deepcopy(self._base_metrics)
        for _gb in self.group_by:
            columns += self.GroupBy[_gb].columns
        columns += TimeUnit[self.time_unit].columns
        return columns

    def get_data_raw(self):
        data = {
            "name": self.name,
            "startDate": self.start_date,
            "endDate": self.end_date,
            "configuration": {
                "adProduct": self._ad_product,
                "groupBy": self.group_by,
                "columns": self.get_columns(),
                "reportTypeId": self._report_type_id,
                "timeUnit": self.time_unit,
                "format": self._format
            }
        }
        return data


class SPCampaignReportsAdapter(AdsV3ReportsAdapter):
    """
    Campaign reports contain performance data broken down at the campaign level.
    Campaign reports include all campaigns of the requested sponsored ad type that have performance activity for the requested days.
    For example, a Sponsored Products campaign report returns performance data for all Sponsored Products campaigns that received impressions on the chosen dates.
    Campaign reports can also be grouped by ad group and placement for more granular data.

    https://advertising.amazon.com/API/docs/en-us/reporting/v3/report-types#campaign-reports
    """

    _report_type_id = "spCampaigns"
    _ad_product = "SPONSORED_PRODUCTS"
    _maximum_date_range = 31
    _data_retention = 95
    _group_by_choices = (
        {"campaign"},
        {"adGroup"},
        {"campaignPlacement"},
        {"campaign", "adGroup"},
        {"campaign", "campaignPlacement"},
        # {"adGroup", "campaignPlacement"},  # FAILED
        # {"campaign", "adGroup", "campaignPlacement"},  # FAILED
    )

    _base_metrics = [
        "impressions",
        "clicks",
        "cost",
        "purchases1d",
        "purchases7d",
        "purchases14d",
        "purchases30d",
        "purchasesSameSku1d",
        "purchasesSameSku7d",
        "purchasesSameSku14d",
        "purchasesSameSku30d",
        "unitsSoldClicks1d",
        "unitsSoldClicks7d",
        "unitsSoldClicks14d",
        "unitsSoldClicks30d",
        "sales1d",
        "sales7d",
        "sales14d",
        "sales30d",
        "attributedSalesSameSku1d",
        "attributedSalesSameSku7d",
        "attributedSalesSameSku14d",
        "attributedSalesSameSku30d",
        "unitsSoldSameSku1d",
        "unitsSoldSameSku7d",
        "unitsSoldSameSku14d",
        "unitsSoldSameSku30d",
        "kindleEditionNormalizedPagesRead14d",
        "kindleEditionNormalizedPagesRoyalties14d",
        # "date",
        # "startDate",
        # "endDate",
        "campaignBiddingStrategy",
        "costPerClick",
        "clickThroughRate",
        "spend",
    ]

    class GroupBy(Enum):

        campaign = [
            "campaignName",
            "campaignId",
            "campaignStatus",
            "campaignBudgetAmount",
            "campaignBudgetType",
            "campaignRuleBasedBudgetAmount",
            "campaignApplicableBudgetRuleId",
            "campaignApplicableBudgetRuleName",
            "campaignBudgetCurrencyCode",
            # "topOfSearchImpressionShare",  # 特殊处理下；仅单独 groupBy campaign 时才会有这个字段
        ]

        adGroup = [
            "adGroupName",
            "adGroupId",
            "adStatus",
        ]

        campaignPlacement = [
            "placementClassification",
        ]

        def __init__(self, columns):
            """Easy dot access like: GroupBy.campaign.columns"""
            self.columns = columns

    def get_data_raw(self):
        columns = self.get_columns()
        if len(self.group_by) == 1 and self.group_by[0] == "campaign":
            columns.append("topOfSearchImpressionShare")

        data = {
            "name": self.name,
            "startDate": self.start_date,
            "endDate": self.end_date,
            "configuration": {
                "adProduct": self._ad_product,
                "groupBy": self.group_by,
                "columns": columns,
                "reportTypeId": self._report_type_id,
                "timeUnit": self.time_unit,
                "format": self._format
            }
        }
        return data


class SPTargetingReportsAdapter(AdsV3ReportsAdapter):
    """
    Targeting reports contain performance metrics broken down by both targeting expressions and keywords.
    To see only targeting expressions, set the keywordType filter to TARGETING_EXPRESSION and TARGETING_EXPRESSION_PREDEFINED.
    To see only keywords, set the keywordType filter to BROAD, PHRASE, and EXACT.

    https://advertising.amazon.com/API/docs/en-us/reporting/v3/report-types#targeting-reports
    """

    _report_type_id = "spTargeting"
    _ad_product = "SPONSORED_PRODUCTS"
    _maximum_date_range = 31
    _data_retention = 95
    _group_by_choices = (
        {"targeting"},
    )

    _base_metrics = [
        'impressions',
        'clicks',
        'costPerClick',
        'clickThroughRate',
        'cost',
        'purchases1d',
        'purchases7d',
        'purchases14d',
        'purchases30d',
        'purchasesSameSku1d',
        'purchasesSameSku7d',
        'purchasesSameSku14d',
        'purchasesSameSku30d',
        'unitsSoldClicks1d',
        'unitsSoldClicks7d',
        'unitsSoldClicks14d',
        'unitsSoldClicks30d',
        'sales1d',
        'sales7d',
        'sales14d',
        'sales30d',
        'attributedSalesSameSku1d',
        'attributedSalesSameSku7d',
        'attributedSalesSameSku14d',
        'attributedSalesSameSku30d',
        'unitsSoldSameSku1d',
        'unitsSoldSameSku7d',
        'unitsSoldSameSku14d',
        'unitsSoldSameSku30d',
        'kindleEditionNormalizedPagesRead14d',
        'kindleEditionNormalizedPagesRoyalties14d',
        'salesOtherSku7d',
        'unitsSoldOtherSku7d',
        'acosClicks7d',
        'acosClicks14d',
        'roasClicks7d',
        'roasClicks14d',
        'keywordId',
        'keyword',
        'campaignBudgetCurrencyCode',
        # 'date',
        # 'startDate',
        # 'endDate',
        'portfolioId',
        'campaignName',
        'campaignId',
        'campaignBudgetType',
        'campaignBudgetAmount',
        'campaignStatus',
        'keywordBid',
        'adGroupName',
        'adGroupId',
        'keywordType',
        'matchType',
        'targeting',

        "topOfSearchImpressionShare",
    ]

    class GroupBy(Enum):

        targeting = [
            "adKeywordStatus",
        ]

        def __init__(self, columns):
            """Easy dot access like: GroupBy.targeting.columns"""
            self.columns = columns


class SPSearchTermReportsAdapter(AdsV3ReportsAdapter):
    """
    Search term reports contain search term performance metrics broken down by targeting expressions and keywords.
    Use the keywordType filter to include either targeting expressions or keywords in your report.

    https://advertising.amazon.com/API/docs/en-us/reporting/v3/report-types#search-term-reports
    """

    _report_type_id = "spSearchTerm"
    _ad_product = "SPONSORED_PRODUCTS"
    _maximum_date_range = 31
    _data_retention = 95
    _group_by_choices = (
        {"searchTerm"},
    )

    _base_metrics = [
        'impressions',
        'clicks',
        'costPerClick',
        'clickThroughRate',
        'cost',
        'purchases1d',
        'purchases7d',
        'purchases14d',
        'purchases30d',
        'purchasesSameSku1d',
        'purchasesSameSku7d',
        'purchasesSameSku14d',
        'purchasesSameSku30d',
        'unitsSoldClicks1d',
        'unitsSoldClicks7d',
        'unitsSoldClicks14d',
        'unitsSoldClicks30d',
        'sales1d',
        'sales7d',
        'sales14d',
        'sales30d',
        'attributedSalesSameSku1d',
        'attributedSalesSameSku7d',
        'attributedSalesSameSku14d',
        'attributedSalesSameSku30d',
        'unitsSoldSameSku1d',
        'unitsSoldSameSku7d',
        'unitsSoldSameSku14d',
        'unitsSoldSameSku30d',
        'kindleEditionNormalizedPagesRead14d',
        'kindleEditionNormalizedPagesRoyalties14d',
        'salesOtherSku7d',
        'unitsSoldOtherSku7d',
        'acosClicks7d',
        'acosClicks14d',
        'roasClicks7d',
        'roasClicks14d',
        'keywordId',
        'keyword',
        'campaignBudgetCurrencyCode',
        # 'date',
        # 'startDate',
        # 'endDate',
        'portfolioId',
        'searchTerm',
        'campaignName',
        'campaignId',
        'campaignBudgetType',
        'campaignBudgetAmount',
        'campaignStatus',
        'keywordBid',
        'adGroupName',
        'adGroupId',
        'keywordType',
        'matchType',
        'targeting',
        # 'adKeywordStatus',
    ]

    class GroupBy(Enum):

        searchTerm = [
            "adKeywordStatus",
        ]

        def __init__(self, columns):
            """Easy dot access like: GroupBy.searchTerm.columns"""
            self.columns = columns


class SPAdvertisedProductReportsAdapter(AdsV3ReportsAdapter):
    """
    Advertised product reports contain performance data for products that are advertised as part of your campaigns.

    https://advertising.amazon.com/API/docs/en-us/reporting/v3/report-types#advertised-product-reports
    """

    _report_type_id = "spAdvertisedProduct"
    _ad_product = "SPONSORED_PRODUCTS"
    _maximum_date_range = 31
    _data_retention = 95
    _group_by_choices = (
        {"advertiser"},
    )

    _base_metrics = [
        # 'date',
        # 'startDate',
        # 'endDate',
        'campaignName',
        'campaignId',
        'adGroupName',
        'adGroupId',
        'adId',
        'portfolioId',
        'impressions',
        'clicks',
        'costPerClick',
        'clickThroughRate',
        'cost',
        'spend',
        'campaignBudgetCurrencyCode',
        'campaignBudgetAmount',
        'campaignBudgetType',
        'campaignStatus',
        'advertisedAsin',
        'advertisedSku',
        'purchases1d',
        'purchases7d',
        'purchases14d',
        'purchases30d',
        'purchasesSameSku1d',
        'purchasesSameSku7d',
        'purchasesSameSku14d',
        'purchasesSameSku30d',
        'unitsSoldClicks1d',
        'unitsSoldClicks7d',
        'unitsSoldClicks14d',
        'unitsSoldClicks30d',
        'sales1d',
        'sales7d',
        'sales14d',
        'sales30d',
        'attributedSalesSameSku1d',
        'attributedSalesSameSku7d',
        'attributedSalesSameSku14d',
        'attributedSalesSameSku30d',
        'salesOtherSku7d',
        'unitsSoldSameSku1d',
        'unitsSoldSameSku7d',
        'unitsSoldSameSku14d',
        'unitsSoldSameSku30d',
        'unitsSoldOtherSku7d',
        'kindleEditionNormalizedPagesRead14d',
        'kindleEditionNormalizedPagesRoyalties14d',
        'acosClicks7d',
        'acosClicks14d',
        'roasClicks7d',
        'roasClicks14d',
    ]

    class GroupBy(Enum):

        advertiser = []

        def __init__(self, columns):
            """Easy dot access like: GroupBy.advertiser.columns"""
            self.columns = columns


class SPPurchasedProductReportsAdapter(AdsV3ReportsAdapter):
    """
    Sponsored Products purchased product reports contain performance data for products that were purchased,
    but were not advertised as part of a campaign.
    The purchased product report contains both targeting expressions and keyword IDs.
    After you have received your report,
    you can filter on keywordType to distinguish between targeting expressions and keywords.

    https://advertising.amazon.com/API/docs/en-us/reporting/v3/report-types#purchased-product-reports
    """

    _report_type_id = "spPurchasedProduct"
    _ad_product = "SPONSORED_PRODUCTS"
    _maximum_date_range = 31
    _data_retention = 95
    _group_by_choices = (
        {"asin"},
    )

    _base_metrics = [
        # 'date',
        # 'startDate',
        # 'endDate',
        'portfolioId',
        'campaignName',
        'campaignId',
        'adGroupName',
        'adGroupId',
        'keywordId',
        'keyword',
        'keywordType',
        'advertisedAsin',
        'purchasedAsin',
        'advertisedSku',
        'campaignBudgetCurrencyCode',
        'matchType',
        'unitsSoldClicks1d',
        'unitsSoldClicks7d',
        'unitsSoldClicks14d',
        'unitsSoldClicks30d',
        'sales1d',
        'sales7d',
        'sales14d',
        'sales30d',
        'purchases1d',
        'purchases7d',
        'purchases14d',
        'purchases30d',
        'unitsSoldOtherSku1d',
        'unitsSoldOtherSku7d',
        'unitsSoldOtherSku14d',
        'unitsSoldOtherSku30d',
        'salesOtherSku1d',
        'salesOtherSku7d',
        'salesOtherSku14d',
        'salesOtherSku30d',
        'purchasesOtherSku1d',
        'purchasesOtherSku7d',
        'purchasesOtherSku14d',
        'purchasesOtherSku30d',
        'kindleEditionNormalizedPagesRead14d',
        'kindleEditionNormalizedPagesRoyalties14d',
    ]

    class GroupBy(Enum):

        asin = []

        def __init__(self, columns):
            """Easy dot access like: GroupBy.asin.columns"""
            self.columns = columns


class SPGrossAndInvalidTrafficReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "spGrossAndInvalids"
    _ad_product = "SPONSORED_PRODUCTS"
    _maximum_date_range = 365
    _data_retention = 365
    _group_by_choices = (
        {"campaign"},
    )
    _format: str = "GZIP_JSON"  # GZIP_JSON or CSV

    _base_metrics = [
        # 'campaignId',  # 不支持
        'campaignName',
        'campaignStatus',
        # 'date',  # 从 timeUnit 里取
        # 'startDate',
        # 'endDate',
        'clicks',
        'impressions',
        'grossClickThroughs',
        'grossImpressions',
        'invalidClickThroughs',
        'invalidClickThroughRate',
        'invalidImpressions',
        'invalidImpressionRate',
    ]

    class GroupBy(Enum):

        campaign = []

        def __init__(self, columns):
            """Easy dot access like: GroupBy.campaign.columns"""
            self.columns = columns


class SBCampaignReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "sbCampaigns"
    _ad_product = "SPONSORED_BRANDS"
    _maximum_date_range = 31
    _data_retention = 60
    _group_by_choices = (
        {"campaign"},
    )

    _base_metrics = [
        'addToCart',
        'addToCartClicks',
        'addToCartRate',
        'brandedSearches',
        'brandedSearchesClicks',
        'campaignBudgetAmount',
        'campaignBudgetCurrencyCode',
        'campaignBudgetType',
        'campaignId',
        'campaignName',
        'campaignStatus',
        'clicks',
        'cost',
        'costType',
        # 'date',
        'detailPageViews',
        'detailPageViewsClicks',
        'eCPAddToCart',
        # 'endDate',
        'impressions',
        'newToBrandDetailPageViewRate',
        'newToBrandDetailPageViews',
        'newToBrandDetailPageViewsClicks',
        'newToBrandECPDetailPageView',
        'newToBrandPurchases',
        'newToBrandPurchasesClicks',
        'newToBrandPurchasesPercentage',
        'newToBrandPurchasesRate',
        'newToBrandSales',
        'newToBrandSalesClicks',
        'newToBrandSalesPercentage',
        'newToBrandUnitsSold',
        'newToBrandUnitsSoldClicks',
        'newToBrandUnitsSoldPercentage',
        'purchases',
        'purchasesClicks',
        'purchasesPromoted',
        'sales',
        'salesClicks',
        'salesPromoted',
        # 'startDate',
        'topOfSearchImpressionShare',
        'unitsSold',
        'unitsSoldClicks',
        'video5SecondViewRate',
        'video5SecondViews',
        'videoCompleteViews',
        'videoFirstQuartileViews',
        'videoMidpointViews',
        'videoThirdQuartileViews',
        'videoUnmutes',
        'viewabilityRate',
        'viewableImpressions',
        'viewClickThroughRate',
    ]

    class GroupBy(Enum):

        # 重复了
        campaign = [
            # "campaignBudgetAmount",
            # "campaignBudgetCurrencyCode",
            # "campaignBudgetType",
            # "topOfSearchImpressionShare",
        ]

        def __init__(self, columns):
            """Easy dot access like: GroupBy.campaign.columns"""
            self.columns = columns


class SBAdGroupReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "sbAdGroup"
    _ad_product = "SPONSORED_BRANDS"
    _maximum_date_range = 31
    _data_retention = 60
    _group_by_choices = (
        {"adGroup"},
        {"campaign"},
    )

    _base_metrics = [
        'addToCart',
        'addToCartClicks',
        'addToCartRate',
        'adGroupId',
        'adGroupName',
        'adStatus',
        'brandedSearches',
        'brandedSearchesClicks',
        'campaignBudgetAmount',
        'campaignBudgetCurrencyCode',
        'campaignBudgetType',
        'campaignId',
        'campaignName',
        'campaignStatus',
        'clicks',
        'cost',
        'costType',
        # 'date',
        'detailPageViews',
        'detailPageViewsClicks',
        'eCPAddToCart',
        # 'endDate',
        'impressions',
        'newToBrandDetailPageViewRate',
        'newToBrandDetailPageViews',
        'newToBrandDetailPageViewsClicks',
        'newToBrandECPDetailPageView',
        'newToBrandPurchases',
        'newToBrandPurchasesClicks',
        'newToBrandPurchasesPercentage',
        'newToBrandPurchasesRate',
        'newToBrandSales',
        'newToBrandSalesClicks',
        'newToBrandSalesPercentage',
        'newToBrandUnitsSold',
        'newToBrandUnitsSoldClicks',
        'newToBrandUnitsSoldPercentage',
        'purchases',
        'purchasesClicks',
        'purchasesPromoted',
        'sales',
        'salesClicks',
        'salesPromoted',
        # 'startDate',
        'unitsSold',
        'unitsSoldClicks',
        'video5SecondViewRate',
        'video5SecondViews',
        'videoCompleteViews',
        'videoFirstQuartileViews',
        'videoMidpointViews',
        'videoThirdQuartileViews',
        'videoUnmutes',
        'viewabilityRate',
        'viewableImpressions',
    ]

    class GroupBy(Enum):

        adGroup = []
        campaign = []

        def __init__(self, columns):
            """Easy dot access like: GroupBy.adGroup.columns"""
            self.columns = columns


class SBPlacementReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "sbCampaignPlacement"
    _ad_product = "SPONSORED_BRANDS"
    _maximum_date_range = 31
    _data_retention = 60
    _group_by_choices = (
        {"campaignPlacement"},
        {"campaign"},
    )

    _base_metrics = [
        'addToCart',
        'addToCartClicks',
        'addToCartRate',
        'brandedSearches',
        'brandedSearchesClicks',
        'campaignBudgetAmount',
        'campaignBudgetCurrencyCode',
        'campaignBudgetType',
        'campaignId',
        'campaignName',
        'campaignStatus',
        'clicks',
        'cost',
        'costType',
        # 'date',
        'detailPageViews',
        'detailPageViewsClicks',
        'eCPAddToCart',
        # 'endDate',
        'impressions',
        'newToBrandDetailPageViewRate',
        'newToBrandDetailPageViews',
        'newToBrandDetailPageViewsClicks',
        'newToBrandECPDetailPageView',
        'newToBrandPurchases',
        'newToBrandPurchasesClicks',
        'newToBrandPurchasesPercentage',
        'newToBrandPurchasesRate',
        'newToBrandSales',
        'newToBrandSalesClicks',
        'newToBrandSalesPercentage',
        'newToBrandUnitsSold',
        'newToBrandUnitsSoldClicks',
        'newToBrandUnitsSoldPercentage',
        'purchases',
        'purchasesClicks',
        'purchasesPromoted',
        'sales',
        'salesClicks',
        'salesPromoted',
        # 'startDate',
        'unitsSold',
        'unitsSoldClicks',
        'video5SecondViewRate',
        'video5SecondViews',
        'videoCompleteViews',
        'videoFirstQuartileViews',
        'videoMidpointViews',
        'videoThirdQuartileViews',
        'videoUnmutes',
        'viewabilityRate',
        'viewableImpressions',
        'viewClickThroughRate'
    ]

    class GroupBy(Enum):

        campaignPlacement = [
            'placementClassification'
        ]
        campaign = []

        def __init__(self, columns):
            """Easy dot access like: GroupBy.campaignPlacement.columns"""
            self.columns = columns


class SBTargetingReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "sbTargeting"
    _ad_product = "SPONSORED_BRANDS"
    _maximum_date_range = 31
    _data_retention = 60
    _group_by_choices = (
        {"targeting"},
    )

    _base_metrics = [
        'addToCart',
        'addToCartClicks',
        'addToCartRate',
        'adGroupId',
        'adGroupName',
        'brandedSearches',
        'brandedSearchesClicks',
        'campaignBudgetAmount',
        'campaignBudgetCurrencyCode',
        'campaignBudgetType',
        'campaignId',
        'campaignName',
        'campaignStatus',
        'clicks',
        'cost',
        'costType',
        # 'date',
        'detailPageViews',
        'detailPageViewsClicks',
        'eCPAddToCart',
        # 'endDate',
        'impressions',
        'keywordBid',
        'keywordId',
        'adKeywordStatus',
        'keywordText',
        'keywordType',
        'matchType',
        'newToBrandDetailPageViewRate',
        'newToBrandDetailPageViews',
        'newToBrandDetailPageViewsClicks',
        'newToBrandECPDetailPageView',
        'newToBrandPurchases',
        'newToBrandPurchasesClicks',
        'newToBrandPurchasesPercentage',
        'newToBrandPurchasesRate',
        'newToBrandSales',
        'newToBrandSalesClicks',
        'newToBrandSalesPercentage',
        'newToBrandUnitsSold',
        'newToBrandUnitsSoldClicks',
        'newToBrandUnitsSoldPercentage',
        'purchases',
        'purchasesClicks',
        'purchasesPromoted',
        'sales',
        'salesClicks',
        'salesPromoted',
        # 'startDate',
        'targetingExpression',
        'targetingId',
        'targetingText',
        'targetingType',
        'topOfSearchImpressionShare',

        'unitsSold',
        'viewableImpressions',
    ]

    class GroupBy(Enum):

        targeting = []

        def __init__(self, columns):
            """Easy dot access like: GroupBy.targeting.columns"""
            self.columns = columns


class SBSearchTermReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "sbSearchTerm"
    _ad_product = "SPONSORED_BRANDS"
    _maximum_date_range = 31
    _data_retention = 60
    _group_by_choices = (
        {"searchTerm"},
    )

    _base_metrics = [
        'adGroupId',
        'adGroupName',
        'campaignBudgetAmount',
        'campaignBudgetCurrencyCode',
        'campaignBudgetType',
        'campaignId',
        'campaignName',
        'campaignStatus',
        'clicks',
        'cost',
        'costType',
        # 'date',
        # 'endDate',
        'impressions',
        'keywordBid',
        'keywordId',
        'keywordText',
        'matchType',
        'purchases',
        'purchasesClicks',
        'sales',
        'salesClicks',
        'searchTerm',
        # 'startDate',
        'unitsSold',
        'video5SecondViewRate',
        'video5SecondViews',
        'videoCompleteViews',
        'videoFirstQuartileViews',
        'videoMidpointViews',
        'videoThirdQuartileViews',
        'videoUnmutes',
        'viewabilityRate',
        'viewableImpressions',
        'viewClickThroughRate'
    ]

    class GroupBy(Enum):

        searchTerm = [
            'adKeywordStatus'
        ]

        def __init__(self, columns):
            """Easy dot access like: GroupBy.searchTerm.columns"""
            self.columns = columns


class SBAdReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "sbAds"
    _ad_product = "SPONSORED_BRANDS"
    _maximum_date_range = 31
    _data_retention = 60
    _group_by_choices = (
        {"ads"},
    )

    _base_metrics = [
        'addToCart',
        'addToCartClicks',
        'addToCartRate',
        'adGroupId',
        'adGroupName',
        'adId',
        'brandedSearches',
        'brandedSearchesClicks',
        'campaignBudgetAmount',
        'campaignBudgetCurrencyCode',
        'campaignBudgetType',
        'campaignId',
        'campaignName',
        'campaignStatus',
        'clicks',
        'cost',
        'costType',
        # 'date',
        'detailPageViews',
        'detailPageViewsClicks',
        'eCPAddToCart',
        # 'endDate',
        'impressions',
        'newToBrandDetailPageViewRate',
        'newToBrandDetailPageViews',
        'newToBrandDetailPageViewsClicks',
        'newToBrandECPDetailPageView',
        'newToBrandPurchases',
        'newToBrandPurchasesClicks',
        'newToBrandPurchasesPercentage',
        'newToBrandPurchasesRate',
        'newToBrandSales',
        'newToBrandSalesClicks',
        'newToBrandSalesPercentage',
        'newToBrandUnitsSold',
        'newToBrandUnitsSoldClicks',
        'newToBrandUnitsSoldPercentage',
        'purchases',
        'purchasesClicks',
        'purchasesPromoted',
        'sales',
        'salesClicks',
        'salesPromoted',
        # 'startDate',
        'unitsSold',
        'unitsSoldClicks',
        'video5SecondViewRate',
        'video5SecondViews',
        'videoCompleteViews',
        'videoFirstQuartileViews',
        'videoMidpointViews',
        'videoThirdQuartileViews',
        'videoUnmutes',
        'viewabilityRate',
        'viewableImpressions'
    ]

    class GroupBy(Enum):

        ads = []

        def __init__(self, columns):
            """Easy dot access like: GroupBy.ads.columns"""
            self.columns = columns


class SBPurchasedProductReportsAdapter(AdsV3ReportsAdapter):
    """
    Sponsored Brands purchased product reports contain performance data for products that were purchased as a result of your campaign.

    https://advertising.amazon.com/API/docs/en-us/reporting/v3/report-types#sponsored-brands
    """

    _report_type_id = "sbPurchasedProduct"
    _ad_product = "SPONSORED_BRANDS"
    _maximum_date_range = 731
    _data_retention = 731
    _group_by_choices = (
        {"purchasedAsin"},
    )

    _base_metrics = [
        # 'date',
        # 'startDate',
        # 'endDate',

        'campaignId',
        'adGroupId',

        'budgetCurrency',
        'campaignBudgetCurrencyCode',
        'campaignName',
        'adGroupName',
        'attributionType',
        'purchasedAsin',
        'productName',
        'productCategory',
        'sales14d',
        'orders14d',
        'unitsSold14d',
        'newToBrandSales14d',
        'newToBrandOrders14d',
        'newToBrandPurchases14d',
        'newToBrandUnitsSold14d',
        'newToBrandSalesPercentage14d',
        'newToBrandOrdersPercentage14d',
        'newToBrandPurchasesPercentage14d',
        'newToBrandUnitsSoldPercentage14d',

        'campaignPriceTypeCode',
        'ordersClicks14d',
        'salesClicks14d',
        'unitsSoldClicks14d',

        # 'viewableImpressions',
    ]

    class GroupBy(Enum):

        purchasedAsin = []

        def __init__(self, columns):
            """Easy dot access like: GroupBy.purchasedAsin.columns"""
            self.columns = columns


class SBGrossAndInvalidTrafficReportsAdapter(SPGrossAndInvalidTrafficReportsAdapter):
    _report_type_id = "sbGrossAndInvalids"
    _ad_product = "SPONSORED_BRANDS"


class SDCampaignReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "sdCampaigns"
    _ad_product = "SPONSORED_DISPLAY"
    _maximum_date_range = 31
    _data_retention = 65
    _group_by_choices = (
        {"campaign"},
        {"matchedTarget"},
        {"campaign", "matchedTarget"},
    )

    _errors_info = {
        'group by campaign & matchedTarget': {
            'code': '400',
            'detail': 'configuration columns include invalid value: (cumulativeReach, impressionsFrequencyAverage, '
                      'newToBrandDetailPageViewClicks, newToBrandDetailPageViewRate, newToBrandDetailPageViews, '
                      'newToBrandDetailPageViewViews, newToBrandECPDetailPageView). '
                      'When matchedTarget is included under groupBy, the following column value cannot be included: ('
                      'impressionsFrequencyAverage, newToBrandDetailPageViews, newToBrandDetailPageViewViews, '
                      'newToBrandDetailPageViewRate, newToBrandDetailPageViewClicks, newToBrandECPDetailPageView, '
                      'cumulativeReach)'
        }
    }

    _base_metrics = [
        'addToCart',
        'addToCartClicks',
        'addToCartRate',
        'addToCartViews',
        'brandedSearches',
        'brandedSearchesClicks',
        'brandedSearchesViews',
        'brandedSearchRate',
        'campaignBudgetCurrencyCode',
        'campaignId',
        'campaignName',
        'clicks',
        'cost',
        # 'date',
        'detailPageViews',
        'detailPageViewsClicks',
        'eCPAddToCart',
        'eCPBrandSearch',
        # 'endDate',
        'impressions',
        'impressionsViews',
        'newToBrandPurchases',
        'newToBrandPurchasesClicks',
        'newToBrandSalesClicks',
        'newToBrandUnitsSold',
        'newToBrandUnitsSoldClicks',
        'purchases',
        'purchasesClicks',
        'purchasesPromotedClicks',
        'sales',
        'salesClicks',
        'salesPromotedClicks',
        # 'startDate',
        'unitsSold',
        'unitsSoldClicks',
        'videoCompleteViews',
        'videoFirstQuartileViews',
        'videoMidpointViews',
        'videoThirdQuartileViews',
        'videoUnmutes',
        'viewabilityRate',
        'viewClickThroughRate',

        'leadFormOpens',
        'leads',
        'linkOuts',
    ]

    class GroupBy(Enum):

        campaign = [
            'campaignBudgetAmount',
            'campaignStatus',
            'costType',
            'cumulativeReach',
            'impressionsFrequencyAverage',
            'newToBrandDetailPageViewClicks',
            'newToBrandDetailPageViewRate',
            'newToBrandDetailPageViews',
            'newToBrandDetailPageViewViews',
            'newToBrandECPDetailPageView',
            'newToBrandSales'
        ]

        matchedTarget = [
            'matchedTargetAsin'
        ]

        def __init__(self, columns):
            """Easy dot access like: GroupBy.campaign.columns"""
            self.columns = columns

    def get_data_raw(self):
        data = super(SDCampaignReportsAdapter, self).get_data_raw()
        if set(self.group_by) == {"campaign", "matchedTarget"}:
            columns = copy.deepcopy(data['configuration']['columns'])
            columns = set(columns) - {
                'cumulativeReach',
                'impressionsFrequencyAverage',
                'newToBrandDetailPageViewClicks',
                'newToBrandDetailPageViewRate',
                'newToBrandDetailPageViews',
                'newToBrandDetailPageViewViews',
                'newToBrandECPDetailPageView'
            }
            data['configuration']['columns'] = list(columns)
        return data


class SDAdGroupReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "sdAdGroup"
    _ad_product = "SPONSORED_DISPLAY"
    _maximum_date_range = 31
    _data_retention = 65
    _group_by_choices = (
        {"adGroup"},
        {"matchedTarget"},
        {"adGroup", "matchedTarget"},
    )

    _errors_info = {
        'group by adGroup and matchedTarget': {
            "code":"400",
            "detail":"configuration columns include invalid value: (cumulativeReach, impressionsFrequencyAverage, "
                     "newToBrandDetailPageViewClicks, newToBrandDetailPageViewRate, newToBrandDetailPageViews, "
                     "newToBrandDetailPageViewViews, newToBrandECPDetailPageView). When matchedTarget is included under "
                     "groupBy, the following column value cannot be included: (impressionsFrequencyAverage, "
                     "newToBrandDetailPageViews, newToBrandDetailPageViewViews, newToBrandDetailPageViewRate, "
                     "newToBrandDetailPageViewClicks, newToBrandECPDetailPageView, cumulativeReach)"
        }
    }

    _base_metrics = [
        'addToCart',
        'addToCartClicks',
        'addToCartRate',
        'addToCartViews',
        'adGroupId',
        'adGroupName',
        'bidOptimization',
        'brandedSearches',
        'brandedSearchesClicks',
        'brandedSearchesViews',
        'brandedSearchRate',
        'campaignBudgetCurrencyCode',
        'campaignId',
        'campaignName',
        'clicks',
        'cost',
        # 'date',
        'detailPageViews',
        'detailPageViewsClicks',
        'eCPAddToCart',
        'eCPBrandSearch',
        # 'endDate',
        'impressions',
        'impressionsViews',
        'newToBrandPurchases',
        'newToBrandPurchasesClicks',
        'newToBrandSales',
        'newToBrandSalesClicks',
        'newToBrandUnitsSold',
        'newToBrandUnitsSoldClicks',
        'purchases',
        'purchasesClicks',
        'purchasesPromotedClicks',
        'sales',
        'salesClicks',
        'salesPromotedClicks',
        # 'startDate',
        'unitsSold',
        'unitsSoldClicks',
        'videoCompleteViews',
        'videoFirstQuartileViews',
        'videoMidpointViews',
        'videoThirdQuartileViews',
        'videoUnmutes',
        'viewabilityRate',
        'viewClickThroughRate',

        'leadFormOpens',
        'leads',
        'linkOuts',
    ]

    class GroupBy(Enum):

        adGroup = [
            'cumulativeReach',
            'impressionsFrequencyAverage',
            'newToBrandDetailPageViewClicks',
            'newToBrandDetailPageViewRate',
            'newToBrandDetailPageViews',
            'newToBrandDetailPageViewViews',
            'newToBrandECPDetailPageView'
        ]

        matchedTarget = [
            'matchedTargetAsin'
        ]

        def __init__(self, columns):
            """Easy dot access like: GroupBy.adGroup.columns"""
            self.columns = columns

    def get_data_raw(self):
        data = super(SDAdGroupReportsAdapter, self).get_data_raw()
        if set(self.group_by) == {"adGroup", "matchedTarget"}:
            columns = copy.deepcopy(data['configuration']['columns'])
            columns = set(columns) - {
                'cumulativeReach',
                'impressionsFrequencyAverage',
                'newToBrandDetailPageViewClicks',
                'newToBrandDetailPageViewRate',
                'newToBrandDetailPageViews',
                'newToBrandDetailPageViewViews',
                'newToBrandECPDetailPageView'
            }
            data['configuration']['columns'] = list(columns)
        return data


class SDTargetingReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "sdTargeting"
    _ad_product = "SPONSORED_DISPLAY"
    _maximum_date_range = 31
    _data_retention = 65
    _group_by_choices = (
        {"targeting"},
        {"matchedTarget"},
        {"targeting", "matchedTarget"},
    )

    _errors_info = {
        'group by adGroup and matchedTarget': {
            "code":"400",
            "detail":"configuration columns include invalid value: (newToBrandDetailPageViewClicks, "
                     "newToBrandDetailPageViewRate, newToBrandDetailPageViews, newToBrandDetailPageViewViews, "
                     "newToBrandECPDetailPageView). When matchedTarget is included under groupBy, the following column "
                     "value cannot be included: (newToBrandDetailPageViews, newToBrandDetailPageViewViews, "
                     "newToBrandDetailPageViewRate, newToBrandDetailPageViewClicks, newToBrandECPDetailPageView)"
        }
    }

    _base_metrics = [
        'addToCart',
        'addToCartClicks',
        'addToCartRate',
        'addToCartViews',
        'adGroupId',
        'adGroupName',
        'brandedSearches',
        'brandedSearchesClicks',
        'brandedSearchesViews',
        'brandedSearchRate',
        'campaignBudgetCurrencyCode',
        'campaignId',
        'campaignName',
        'clicks',
        'cost',
        # 'date',
        'detailPageViews',
        'detailPageViewsClicks',
        'eCPAddToCart',
        'eCPBrandSearch',
        # 'endDate',
        'impressions',
        'impressionsViews',
        'newToBrandPurchases',
        'newToBrandPurchasesClicks',
        'newToBrandSales',
        'newToBrandSalesClicks',
        'newToBrandUnitsSold',
        'newToBrandUnitsSoldClicks',
        'purchases',
        'purchasesClicks',
        'purchasesPromotedClicks',
        'sales',
        'salesClicks',
        'salesPromotedClicks',
        # 'startDate',
        'targetingExpression',
        'targetingId',
        'targetingText',
        'unitsSold',
        'unitsSoldClicks',
        'videoCompleteViews',

        'leadFormOpens',
        'leads',
        'linkOuts',
        'videoFirstQuartileViews',
        'videoMidpointViews',
        'videoThirdQuartileViews',
        'videoUnmutes',
        'viewClickThroughRate',
        'viewabilityRate',
    ]

    class GroupBy(Enum):

        targeting = [
            'adKeywordStatus',
            'newToBrandDetailPageViewClicks',
            'newToBrandDetailPageViewRate',
            'newToBrandDetailPageViews',
            'newToBrandDetailPageViewViews',
            'newToBrandECPDetailPageView'
        ]

        matchedTarget = [
            'matchedTargetAsin'
        ]

        def __init__(self, columns):
            """Easy dot access like: GroupBy.targeting.columns"""
            self.columns = columns

    def get_data_raw(self):
        data = super(SDTargetingReportsAdapter, self).get_data_raw()
        if set(self.group_by) == {"targeting", "matchedTarget"}:
            columns = copy.deepcopy(data['configuration']['columns'])
            columns = set(columns) - {
                'newToBrandDetailPageViewClicks',
                'newToBrandDetailPageViewRate',
                'newToBrandDetailPageViews',
                'newToBrandDetailPageViewViews',
                'newToBrandECPDetailPageView'
            }
            data['configuration']['columns'] = list(columns)
        return data


class SDAdvertisedProductReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "sdAdvertisedProduct"
    _ad_product = "SPONSORED_DISPLAY"
    _maximum_date_range = 31
    _data_retention = 65
    _group_by_choices = (
        {"advertiser"},
    )

    _base_metrics = [
        'addToCart',
        'addToCartClicks',
        'addToCartRate',
        'addToCartViews',
        'adGroupId',
        'adGroupName',
        'adId',
        'bidOptimization',
        'brandedSearches',
        'brandedSearchesClicks',
        'brandedSearchesViews',
        'brandedSearchRate',
        'campaignBudgetCurrencyCode',
        'campaignId',
        'campaignName',
        'clicks',
        'cost',
        'cumulativeReach',
        # 'date',
        'detailPageViews',
        'detailPageViewsClicks',
        'eCPAddToCart',
        'eCPBrandSearch',
        # 'endDate',
        'impressions',
        'impressionsFrequencyAverage',
        'impressionsViews',
        'newToBrandDetailPageViewClicks',
        'newToBrandDetailPageViewRate',
        'newToBrandDetailPageViews',
        'newToBrandDetailPageViewViews',
        'newToBrandECPDetailPageView',
        'newToBrandPurchases',
        'newToBrandPurchasesClicks',
        'newToBrandSales',
        'newToBrandSalesClicks',
        'newToBrandUnitsSold',
        'newToBrandUnitsSoldClicks',
        'promotedAsin',
        'promotedSku',
        'purchases',
        'purchasesClicks',
        'purchasesPromotedClicks',
        'sales',
        'salesClicks',
        'salesPromotedClicks',
        # 'startDate',
        'unitsSold',
        'unitsSoldClicks',
        'videoCompleteViews',
        'videoFirstQuartileViews',
        'videoMidpointViews',
        'videoThirdQuartileViews',
        'videoUnmutes',
        'viewabilityRate',
        'viewClickThroughRate',

        'leadFormOpens',
        'leads',
        'linkOuts',
    ]

    class GroupBy(Enum):

        advertiser = []

        def __init__(self, columns):
            """Easy dot access like: GroupBy.advertiser.columns"""
            self.columns = columns


class SDPurchasedProductReportsAdapter(AdsV3ReportsAdapter):
    _report_type_id = "sdPurchasedProduct"
    _ad_product = "SPONSORED_DISPLAY"
    _maximum_date_range = 31
    _data_retention = 65
    _group_by_choices = (
        {"asin"},
    )

    _base_metrics = [
        'adGroupId',
        'adGroupName',
        'asinBrandHalo',
        'campaignBudgetCurrencyCode',
        'campaignId',
        'campaignName',
        'conversionsBrandHalo',
        'conversionsBrandHaloClicks',
        # 'date',
        # 'endDate',
        'promotedAsin',
        'promotedSku',
        'salesBrandHalo',
        'salesBrandHaloClicks',
        # 'startDate',
        'unitsSoldBrandHalo',
        'unitsSoldBrandHaloClicks'
    ]

    class GroupBy(Enum):

        asin = []

        def __init__(self, columns):
            """Easy dot access like: GroupBy.asin.columns"""
            self.columns = columns


class SDGrossAndInvalidTrafficReportsAdapter(SPGrossAndInvalidTrafficReportsAdapter):
    _report_type_id = "sdGrossAndInvalids"
    _ad_product = "SPONSORED_DISPLAY"


if __name__ == '__main__':

    # pprint(SPCampaignReportsAdapter(group_by=["campaign"], time_unit="DAILY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["campaign"], time_unit="SUMMARY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["adGroup"], time_unit="DAILY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["adGroup"], time_unit="SUMMARY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["campaignPlacement"], time_unit="DAILY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["campaignPlacement"], time_unit="SUMMARY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["campaign", "adGroup"], time_unit="DAILY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["campaign", "adGroup"], time_unit="SUMMARY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["campaign", "campaignPlacement"], time_unit="DAILY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["campaign", "campaignPlacement"], time_unit="SUMMARY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    #
    # TODO FAILED !!! failureReason: Report generation failed due to an internal error. Please retry
    # pprint(SPCampaignReportsAdapter(group_by=["adGroup", "campaignPlacement"], time_unit="DAILY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["adGroup", "campaignPlacement"], time_unit="SUMMARY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["campaign", "adGroup", "campaignPlacement"], time_unit="DAILY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPCampaignReportsAdapter(group_by=["campaign", "adGroup", "campaignPlacement"], time_unit="SUMMARY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    #
    # pprint(SPTargetingReportsAdapter(group_by=["targeting"], time_unit="DAILY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPTargetingReportsAdapter(group_by=["targeting"], time_unit="SUMMARY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    #
    # pprint(SPSearchTermReportsAdapter(group_by=["searchTerm"], time_unit="DAILY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPSearchTermReportsAdapter(group_by=["searchTerm"], time_unit="SUMMARY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    #
    # pprint(SPAdvertisedProductReportsAdapter(group_by=["advertiser"], time_unit="DAILY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPAdvertisedProductReportsAdapter(group_by=["advertiser"], time_unit="SUMMARY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    #
    # pprint(SPPurchasedProductReportsAdapter(group_by=["asin"], time_unit="DAILY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    # pprint(SPPurchasedProductReportsAdapter(group_by=["asin"], time_unit="SUMMARY", start_date="2022-07-01", end_date="2022-07-31").get_data_raw())
    #
    # pprint(SPGrossAndInvalidTrafficReportsAdapter(group_by=["campaign"], time_unit="DAILY", start_date="2023-01-01", end_date="2023-10-01").get_data_raw())
    # pprint(SPGrossAndInvalidTrafficReportsAdapter(group_by=["campaign"], time_unit="SUMMARY", start_date="2023-01-01", end_date="2023-10-01").get_data_raw())
    #
    #
    # pprint(SBCampaignReportsAdapter(group_by=["campaign"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SBCampaignReportsAdapter(group_by=["campaign"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    #
    # pprint(SBPurchasedProductReportsAdapter(group_by=["purchasedAsin"], time_unit="DAILY", start_date="2024-07-01", end_date="2024-07-10").get_data_raw())
    # pprint(SBPurchasedProductReportsAdapter(group_by=["purchasedAsin"], time_unit="SUMMARY", start_date="2021-07-01", end_date="2022-07-01").get_data_raw())
    #
    # pprint(SBGrossAndInvalidTrafficReportsAdapter(group_by=["campaign"], time_unit="DAILY", start_date="2023-01-01", end_date="2023-10-01").get_data_raw())
    # pprint(SBGrossAndInvalidTrafficReportsAdapter(group_by=["campaign"], time_unit="SUMMARY", start_date="2023-01-01", end_date="2023-10-01").get_data_raw())
    #
    #
    # pprint(SDCampaignReportsAdapter(group_by=["campaign"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDCampaignReportsAdapter(group_by=["campaign"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDCampaignReportsAdapter(group_by=["matchedTarget"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDCampaignReportsAdapter(group_by=["matchedTarget"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDCampaignReportsAdapter(group_by=["campaign", "matchedTarget"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDCampaignReportsAdapter(group_by=["campaign", "matchedTarget"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())

    # pprint(SDAdGroupReportsAdapter(group_by=["adGroup"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDAdGroupReportsAdapter(group_by=["adGroup"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDAdGroupReportsAdapter(group_by=["matchedTarget"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDAdGroupReportsAdapter(group_by=["matchedTarget"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDAdGroupReportsAdapter(group_by=["adGroup", "matchedTarget"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDAdGroupReportsAdapter(group_by=["adGroup", "matchedTarget"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())

    # pprint(SDTargetingReportsAdapter(group_by=["targeting"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDTargetingReportsAdapter(group_by=["targeting"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDTargetingReportsAdapter(group_by=["matchedTarget"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDTargetingReportsAdapter(group_by=["matchedTarget"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDTargetingReportsAdapter(group_by=["targeting", "matchedTarget"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDTargetingReportsAdapter(group_by=["targeting", "matchedTarget"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())

    # pprint(SDAdvertisedProductReportsAdapter(group_by=["advertiser"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDAdvertisedProductReportsAdapter(group_by=["advertiser"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())

    # pprint(SDPurchasedProductReportsAdapter(group_by=["asin"], time_unit="DAILY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())
    # pprint(SDPurchasedProductReportsAdapter(group_by=["asin"], time_unit="SUMMARY", start_date="2023-11-01", end_date="2023-11-30").get_data_raw())

    # pprint(SDGrossAndInvalidTrafficReportsAdapter(group_by=["campaign"], time_unit="DAILY", start_date="2023-01-01", end_date="2023-10-01").get_data_raw())
    # pprint(SDGrossAndInvalidTrafficReportsAdapter(group_by=["campaign"], time_unit="SUMMARY", start_date="2023-01-01", end_date="2023-10-01").get_data_raw())

    # pprint(SBTargetingReportsAdapter(group_by=["targeting"], time_unit="DAILY", start_date="2024-07-01", end_date="2024-07-10").get_data_raw())
    # pprint(SBAdGroupReportsAdapter(group_by=["adGroup"], time_unit="DAILY", start_date="2024-07-01", end_date="2024-07-02").get_data_raw())
    pprint(SBPurchasedProductReportsAdapter(group_by=["purchasedAsin"], time_unit="DAILY", start_date="2024-07-01", end_date="2024-07-02").get_data_raw())

    # response = ads_request_report(payload, ad_type='sb', record_type='c_campaign_daily', start_date='2024-06-16', end_date='2024-07-17')
    # response = ads_request_report(payload, ad_type='sb', record_type='c_campaign_summary', start_date='2024-06-16', end_date='2024-07-17')
    #
    # response = ads_request_report(payload, ad_type='sb', record_type='cp_campaign_placement_daily', start_date='2024-06-16', end_date='2024-07-17')
    # response = ads_request_report(payload, ad_type='sb', record_type='cp_campaign_placement_summary', start_date='2024-06-16', end_date='2024-07-17')
    #
    # response = ads_request_report(payload, ad_type='sb', record_type='ag_ad_group_daily', start_date='2024-06-16', end_date='2024-07-17')
    # response = ads_request_report(payload, ad_type='sb', record_type='ag_ad_group_summary', start_date='2024-06-16', end_date='2024-07-17')
    #
    # response = ads_request_report(payload, ad_type='sb', record_type='a_ads_daily', start_date='2024-06-16', end_date='2024-07-17')
    # response = ads_request_report(payload, ad_type='sb', record_type='a_ads_summary', start_date='2024-06-16', end_date='2024-07-17')
    #
    # response = ads_request_report(payload, ad_type='sb', record_type='t_targeting_daily', start_date='2024-06-16', end_date='2024-07-17')
    # response = ads_request_report(payload, ad_type='sb', record_type='t_targeting_summary', start_date='2024-06-16', end_date='2024-07-17')
    #
    # response = ads_request_report(payload, ad_type='sb', record_type='st_search_term_daily', start_date='2024-06-16', end_date='2024-07-17')
    # response = ads_request_report(payload, ad_type='sb', record_type='st_search_term_summary', start_date='2024-06-16', end_date='2024-07-17')
    #
    #
    # response = ads_request_report(payload, ad_type='sd', record_type='c_campaign_matched_target_daily', start_date='2024-06-16', end_date='2024-07-17')
    # response = ads_request_report(payload, ad_type='sd', record_type='c_campaign_matched_target_summary', start_date='2024-06-16', end_date='2024-07-17')
    #
    # response = ads_request_report(payload, ad_type='sd', record_type='ag_ad_group_matched_target_daily', start_date='2024-06-16', end_date='2024-07-17')
    # response = ads_request_report(payload, ad_type='sd', record_type='ag_ad_group_matched_target_summary', start_date='2024-06-16', end_date='2024-07-17')
    #
    # response = ads_request_report(payload, ad_type='sd', record_type='t_targeting_matched_target_daily', start_date='2024-06-16', end_date='2024-07-17')
    # response = ads_request_report(payload, ad_type='sd', record_type='t_targeting_matched_target_summary', start_date='2024-06-16', end_date='2024-07-17')
    #
    # response = ads_request_report(payload, ad_type='sd', record_type='pp_asin_daily', start_date='2024-06-16', end_date='2024-07-17')
    # response = ads_request_report(payload, ad_type='sd', record_type='pp_asin_summary', start_date='2024-06-16', end_date='2024-07-17')
    #

